/**
 * @name	SlideRefresh
 * @version	20140825_1
 * @since	201408
 * @param1 $element, 如 $("#main")
 * @param2 Object	opts
 * bounceTime	-	600：超出边界回弹时间
 * isTopRefresh  -   true: 是否下拉刷新
 * decelerationVal - 0.002: 屏幕滑动停止的加速度值（越大，停得越快）
 * hasScrollBar - true: 是否有滚动条
 * isBottomRefresh - true: 是否上拉刷新
 * topTips / bottomTips -  Object: 
 * 		tips: 刷新提示语html代码内容
 * 		tips1：提示语刚开始拉动时执行的函数
 * 		tips2：提示语拉动位置超过可刷新位置时执行的函数
 * 		tips3：放开拉动且可刷新时执行的函数
 * 		content: 拉动部分的内容代码
 * 		tipsHeight：提示语的高度，也是是否刷新的高度
 * 		appear: 插入（上/下）html后执行的函数，一般用于修改样式
 * 				参数是（上/下）添加的$panel
 *topLoadFunc / bottomLoadFunc - function : 刷新执行函数，
 *							参数为 刷新后隐藏loading提示语的回调函数
 * 
 * @example
	var iscroll = new SlideRefresh($("#main"));
	iscroll.init();
*/
function SlideRefresh(el,opts){
	this.wrapper = el;
	this.scroller = $(this.wrapper).children()[0];
	this.opts = opts;

	this.opts = jQuery.extend(true,{
		bounceTime : 600,
		decelerationVal: 0.002,
		isTopRefresh: true,
		isBottomRefresh: true,
		hasScrollBar: true,
		topTips: { //下拉刷新的参数
			tips: "<span class='icon-arrow-down'></span><p>下拉可以刷新...</p>",
			tips1: function($el){$el.find('p').html("下拉可以刷新...");$el.find('span').removeClass('pull').removeClass('icon-spinner').addClass('icon-arrow-down');},
			tips2: function($el){$el.find('p').html("放手即刷新...");$el.find('span').addClass('pull');},
			tips3: function($el){$el.html("<span class='icon-spinner'></span><p>正在加载中...</p>");},
			content: '<div style="height:100px;background:#ccc;"></div>',
			tipsHeight: 40,
			appear: $.noop
		},
		bottomTips: { //上拉刷新的参数
			tips: "<span class='icon-arrow-up'></span><p>上拉可以刷新...</p>",
			tips1: function($el){$el.find('p').html("上拉可以刷新...");$el.find('span').removeClass('pull').removeClass('icon-spinner').addClass('icon-arrow-up');},
			tips2: function($el){$el.find('p').html("放手即刷新...");$el.find('span').addClass('pull');},
			tips3: function($el){$el.html("<span class='icon-spinner'></span><p>正在加载中...</p>");},
			content: '<div style="height:100px;background:#ccc;"></div>',
			tipsHeight: 40,
			appear: $.noop
		},
		topLoadFunc: function(cb){ //下拉刷新后的回调函数
			setTimeout(cb,2000);
		},
		bottomLoadFunc: function(cb){ //上拉刷新后的回调函数
			setTimeout(cb,2000);
		}
	}, opts);
	this.supportsCssTransitions = (function (style) {
		var prefixes = ['Webkit','Moz','Ms'];
		for(var i=0, l=prefixes.length; i < l; i++ ) {
			if( typeof style[prefixes[i] + 'Transition'] !== 'undefined') {
				return prefixes[i].toLowerCase();
			}
		}
		return false;
	})(document.createElement('div').style);
}

var rsUtils = {
	defaultPoint: [{pageY: 0, pageX: 0}],
	easing: {
		quadratic: {
			style: 'cubic-bezier(0.25, 0.46, 0.45, 0.94)',
			fn: function (k) {
				return k * ( 2 - k );
			}
		},
		circular: {
			style: 'cubic-bezier(0.1, 0.57, 0.1, 1)',	// Not properly "circular" but this looks better, it should be (0.075, 0.82, 0.165, 1)
			fn: function (k) {
				return Math.sqrt( 1 - ( --k * k ) );
			}
		},
		back: {
			style: 'cubic-bezier(0.175, 0.885, 0.32, 1.275)',
			fn: function (k) {
				var b = 4;
				return ( k = k - 1 ) * k * ( ( b + 1 ) * k + b ) + 1;
			}
		}
	},
	momentum : function (current, start, time, lowerMargin, higherMargin, wrapperSize, deceleration) {
		if(!time){
			return {
				destination: Math.round(current),
				duration: 0
			};
		}
		var MAX_SPEED = 2;
		var distance = current - start,
			speed = Math.abs(distance) / time,
			destination,
			duration;

		speed = speed > MAX_SPEED ? MAX_SPEED : speed;
		deceleration = deceleration === undefined ? 0.0006 : deceleration;

		destination = current + ( speed * speed ) / ( 2 * deceleration ) * ( distance < 0 ? -1 : 1 );
		duration = speed / deceleration;

		if ( destination < lowerMargin ) {
			destination = wrapperSize ? lowerMargin - ( wrapperSize / 2.5 * ( speed / 8 ) ) : lowerMargin;
			distance = Math.abs(destination - current);
			duration = distance / speed;
		} else if ( destination > higherMargin ) {
			destination = wrapperSize ? wrapperSize / 2.5 * ( speed / 8 ) : 0;
			distance = Math.abs(current) + destination;
			duration = distance / speed;
		}

		return {
			destination: Math.round(destination),
			duration: duration
		}
	},
	prefixPointerEvent : function (pointerEvent) {
		return window.MSPointerEvent ? 
			'MSPointer' + pointerEvent.charAt(9).toUpperCase() + pointerEvent.substr(10):
			pointerEvent;
	},
	hasPoint: function(){
		return (window.PointerEvent || window.MSPointerEvent);
	},
	hasTouch: function(){
		return 'ontouchstart' in window;
	}
};

var barTimer = null, scrollTimer = null;
SlideRefresh.prototype = {
	init: function(){
		this.wrapperHeight = $(this.wrapper).height();
		this.scrollerHeight = $(this.scroller).height();
		(this.scrollerHeight < this.wrapperHeight) && (this.scrollerHeight = this.wrapperHeight);
		this.lowerMargin = this.wrapperHeight - this.scrollerHeight;
		this.higherMargin = 0;
		$(this.scroller).css({
			position: 'absolute',
			top: 0,
			width: '100%'
		});
		if(this.opts.isTopRefresh){
			this.$topPanel = this.addPanel('top');
			this.opts.topTips.appear.call(this, this.$topPanel);
		}
		if(this.opts.isBottomRefresh){
			this.$bottomPanel = this.addPanel('bottom');
			this.opts.bottomTips.appear.call(this, this.$bottomPanel);
		}
		this._curHeight = 0;
		this.topLoading = false;
		this.bottomLoading = false;
		this.moving = false;

		if(this.opts.hasScrollBar){
			this.$scrollBar = $('<div class="scrollBar"></div>');
			this.$scrollBar.css({
				position: 'absolute',
				right: '1px',
				top: 0,
				width: '4px',
				background: "#000",
				opacity: 0.3,
				'border-radius': '1px',
				display: 'none'
			});
			$(this.scroller).after(this.$scrollBar);
			this.setScrollBar();
		}
		this._initEvents();
	},
	_initEvents: function(){
		if(rsUtils.hasTouch()){
			$(this.wrapper)
				.bind("touchstart", $.proxy(this.touchstart,this))
				.bind("touchmove", $.proxy(this.touchmove,this))
				.bind("touchcancel",$.proxy(this.touchend,this))
				.bind("touchend", $.proxy(this.touchend,this));
		}else{
			$(this.wrapper)
				.bind("mousedown", $.proxy(this.touchstart,this))
				.bind("mousemove", $.proxy(this.touchmove,this))
				.bind("mousecancel",$.proxy(this.touchend,this))
				.bind("mouseup", $.proxy(this.touchend,this));
		}
		if(rsUtils.hasPoint()){
			$(this.wrapper)
				.bind(rsUtils.prefixPointerEvent('pointerdown'), $.proxy(this.touchstart,this))
				.bind(rsUtils.prefixPointerEvent('pointermove'), $.proxy(this.touchmove,this))
				.bind(rsUtils.prefixPointerEvent('pointercancel'), $.proxy(this.touchend,this))
				.bind(rsUtils.prefixPointerEvent('pointerup'), $.proxy(this.touchend,this));
		}
		$(window).resize($.proxy(this.checkHeight,this));
	},
	addPanel: function(type){
		var typeCnt = {
			top: {
				cnt: this.opts.topTips,
				add: 'prepend',
				style: {
					position: 'absolute',
					width: '100%',
					bottom: 0
				}
			},
			bottom: {
				cnt: this.opts.bottomTips,
				add: 'append',
				style: {
					position: 'absolute',
					width: '100%',
					top: 0
				}
			}
		};
		var curType = typeCnt[type];
		var addCnt = '<div class="rf_'+ type +'"><div class="rf_cnt">' + curType.cnt.content 
			+ '</div><div class="rf_tips">' + curType.cnt.tips + '</div></div>';
		var $panel = $(addCnt);

		$panel.find(".rf_tips").css({
			height: curType.cnt.tipsHeight + "px",
			'line-height': curType.cnt.tipsHeight + "px",
			width: '100%',
			'text-align': 'center'
		});
		$panel.find(".rf_tips,.rf_cnt").css(curType.style);
		$(this.scroller)[curType.add]($panel);
		var maxHeight = $panel.find(".rf_cnt").height();
		maxHeight < curType.cnt.tipsHeight && (maxHeight = curType.cnt.tipsHeight);
		$panel.css({
			position: "absolute",
			width: "100%",
			"height": maxHeight  + 'px',
			'z-index': 100,
			'clear': 'both',
			"overflow": "hidden"
		});
		var panelPos = {
			top: {top: -maxHeight + 'px'},
			bottom: {top: this.scrollerHeight + 'px'}
		};
		$panel.css(panelPos[type]);
		return $panel;
	},
	setScrollBar: function(time){
		var time = time || 0;
		this.$scrollBar.stop();
		clearTimeout(barTimer);
		if(!this.$scrollBar){ return; }
		if(this.wrapperHeight < this.scrollerHeight){
			var height = this.wrapperHeight / this.scrollerHeight * this.wrapperHeight;
			var top = - this._curHeight * this.wrapperHeight / this.scrollerHeight;
			if(top < 0){
				height = height + top;
				height = (height < 8 ? 8 : height);
				top = 0;
			}
			if(height + top > this.wrapperHeight){
				height = this.wrapperHeight - top;
				height = (height < 8 ? 8 : height);
				top = this.wrapperHeight - height;
			}
			var _this = this;
			var showTime = 50;
			this.$scrollBar.css({
				'-webkit-transition': time + 'ms',
				height: height + 'px',
				top : top + 'px'
			}).fadeIn(showTime,function(){
				_this.hideBar.call(_this,showTime);
			});
		}else{
			this.$scrollBar.fadeOut(showTime);
		}
	},
	hideBar: function(showTime){
		var _t = this;
		clearTimeout(barTimer);
		barTimer = setTimeout(function(){
			if(_t.moving){
				_t.hideBar();
			}else{
				_t.$scrollBar.fadeOut(showTime);
			}
		},500);
	},
	touchstart: function(e){
		var point = e.touches || e.originalEvent.touches || rsUtils.defaultPoint;
		if((e.type == "touchstart" && point.length <= 1) || e.type == "dragstart") {
			if(this.supportsCssTransitions) {
				var trans = rsUtils.easing.circular.style + " 0ms";
				$(this.scroller).css({
					"-moz-transition" : trans,
					"-ms-transition" : trans,
					"-webkit-transition" : trans,
					"transition" : trans
				});
			}
			if(this.moving){
				var pos = this.getComputedPosition();
				this._curHeight = pos.y;
				this._translate();
				this.moving = false;
				$(this.scroller).unbind("webkitTransitionEnd");
			}
			this.checkHeight();
			this._startY = e.pageY || (point[0].pageY) || 0;
			this._cur = this._startY;
			this._startTime = e.timeStamp || new Date().getTime();
			this._endTime = this._startTime;
		}
	},
	touchmove: function(e){
		var point = e.touches || e.originalEvent.touches || rsUtils.defaultPoint;
		var curP = e.pageY || (point[0].pageY) || 0;
		if((e.type == "touchmove" && point.length <= 1) || e.type == "drag") {
			this._startTime = this._endTime;
			this._endTime = e.timeStamp || new Date().getTime();
			if(this._endTime - this._startTime > 300 && curP - this._cur < 10){
				return;
			}
			this._startY = this._cur;
			this._cur = curP;
			if(this._cur != this._startY){e.preventDefault();}
			var detaY = this._cur - this._startY;
			this._move(detaY);
			this.moving = true;
			if(this.supportsCssTransitions) {
				var trans = "translate3d(0," + this._curHeight + "px,0)";
				$(this.scroller).css({
					"-moz-transform" : trans,
					"-ms-transform" : trans,
					"-webkit-transform" : trans,
					"transform" : trans
				});
			} else {
				$(this.scroller).animate({
					"top": this._curHeight
				},0);
			}
			if(!this.topLoading){
				if(this._curHeight > this.higherMargin + this.opts.topTips.tipsHeight){
					this._tipsChange("top", 2);
				}else{
					this._tipsChange("top", 1);
				}
			}
			if(!this.bottomLoading){
				if(this._curHeight < this.lowerMargin - this.opts.bottomTips.tipsHeight){
					this._tipsChange("bottom", 2);
				}else{
					this._tipsChange("bottom", 1);
				}
			}
			this.setScrollBar();
		}
	},
	touchend : function (e) {
		var point = e.touches || e.originalEvent.touches;
		this.moving = false;
		if((e.type == "touchend" && point.length <= 1) || e.type == "dragend") {
			//this._endTime = e.timeStamp || new Date().getTime();
			var time = this._endTime - this._startTime;
			var start = this._curHeight - (this._cur - this._startY);
			
			this._animate(time,start);
		}
	},
	_move : function (deta) {
		var deta = deta || 0;
		var tmp = this._curHeight + deta;
		if(tmp < this.lowerMargin || tmp > this.higherMargin){
			this._curHeight = this._curHeight + deta / 3;
		}else{
			this._curHeight = tmp;
		}
	},

	_tipsChange: function(type, step){
		var $el = (type == "top" ? this.$topPanel : this.$bottomPanel);
		if(!$el){return;}
		var func = (type == "top" ? this.opts.topTips : this.opts.bottomTips);
		var s = "tips" + step;
		func[s] && func[s]($el.find(".rf_tips"));
	},
	
	_animate : function (time,start) {
		if(this.resetPosition(this.opts.bounceTime)){
			return;
		}
		var duration = 0;
		if(time < 300){
			var momentumY = rsUtils.momentum(this._curHeight, start, time, this.lowerMargin, this.higherMargin, this.wrapperHeight / 4,this.opts.decelerationVal);
			this._curHeight = momentumY.destination || this._curHeight;
			duration = momentumY.duration;
		}

		this._translate(duration);
	},

	_translate: function(time){
		this.moving = true;
		var _t = this;
		if(this.supportsCssTransitions) {
			var transition = rsUtils.easing.circular.style + ' ' + time + "ms";
			var transform = "translate3d(0," + this._curHeight + "px,0)";
			$(this.scroller).one("webkitTransitionEnd",function(){
				_t.translateEnd.call(_t,false);
			});
			$(this.scroller).css({
				"-moz-transition" : transition,
				"-moz-transform" : transform,
				"-ms-transition" : transition,
				"-ms-transform" : transform,
				"-webkit-transition" : transition,
				"-webkit-transform" : transform,
				"transition" : transition,
				"transform" : transform
			});
			
		} else {console.log(time);
			$(this.scroller).stop();
			$(this.scroller).animate({"top": this._curHeight}, time, function(){
					_t.translateEnd.call(_t,false);
				});
		}
		this.setScrollBar(time);
	},

	translateEnd: function(hasReset){
		this.moving = false;
		if(this._curHeight - this._cur != 0){
			clearTimeout(scrollTimer);
			scrollTimer = setTimeout(function(){
				$(window).trigger("scroll");
			},100);
		}
		if(!hasReset){
			this.resetPosition(this.opts.bounceTime);
		}
	},

	resetPosition: function(time){
		if(this.moving){return false;}
		var time = time || 0;
		if(!this.topLoading && this.opts.isTopRefresh){
			var topH = this.higherMargin + this.opts.topTips.tipsHeight;
			if(this._curHeight > topH){
				this.higherMargin = topH;
				this._tipsChange("top",3);
				this.topLoading = true;
				this.opts.topLoadFunc && this.opts.topLoadFunc($.proxy(this.topDone,this));
			}
		}
		if(!this.bottomLoading && this.opts.isBottomRefresh){
			var bottomH = this.lowerMargin - this.opts.bottomTips.tipsHeight;
			if(this._curHeight < bottomH){
				this.lowerMargin = bottomH;
				this._tipsChange("bottom",3);
				this.bottomLoading = true;
				this.opts.bottomLoadFunc && this.opts.bottomLoadFunc($.proxy(this.bottomDone,this));
			}
		}
		if(this._curHeight > this.higherMargin){
			this._curHeight = this.higherMargin;
		}else if(this._curHeight < this.lowerMargin){
			this._curHeight = this.lowerMargin;
		}else{
			this.translateEnd(true);
			return false;
		}
		this._translate(time);
		return true;
	},
	getComputedPosition: function () {
		var matrix = window.getComputedStyle(this.scroller, null),
			x, y;

		if ( this.supportsCssTransitions ) {
			matrix = matrix[this.supportsCssTransitions + 'Transform'].split(')')[0].split(', ');
			x = +(matrix[12] || matrix[4]);
			y = +(matrix[13] || matrix[5]);
		} else {
			x = +matrix.left.replace(/[^-\d.]/g, '');
			y = +matrix.top.replace(/[^-\d.]/g, '');
		}

		return { x: x, y: y };
	},
	checkHeight: function(){
		var h = $(this.scroller).height();
		var wH = $(this.wrapper).height();
		if(h < wH){h = wH;}
		if(h !== this.scrollerHeight || wH !== this.wrapperHeight){
			var delta = (wH - h) - this.lowerMargin;
			this.lowerMargin = this.lowerMargin + delta;
			this.scrollerHeight = h;
			this.wrapperHeight = wH;
			this.setScrollBar();
			if(this.$bottomPanel.length){
				this.$bottomPanel.css({top: this.scrollerHeight + 'px'});
			}
		}
	},
	topDone: function(){
		this.higherMargin = this.higherMargin - this.opts.topTips.tipsHeight;
		if(this._curHeight > this.higherMargin){
			this._curHeight = this.higherMargin;
			this.setScrollBar();
		}
		this.topLoading = false;
		this._translate(this.opts.bounceTime);
	},
	bottomDone: function(){
		this.lowerMargin = this.lowerMargin + this.opts.bottomTips.tipsHeight;
		if(this._curHeight < this.lowerMargin){
			this._curHeight = this.lowerMargin;
			this.setScrollBar();
		}
		this.bottomLoading = false;
		this._translate(this.opts.bounceTime);
	}
};

